/* sprsch.c
 * 
 * Copyright (C) 2002 Claudio Girardi  <in3otd@qsl.net>
 * 
 * This file was derived from the original Pascal code, written by
 * Stephen A. Maas. It was translated into ANSI C with the help of p2c,
 * version 1.21alpha, and further manually refined by Claudio Girardi.
 * 
 * Bugs were certainly introduced during the translation process, so
 * please don't bother Sthephen with bug reports but send them to
 * Claudio, thanks.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA, 02111-1307, USA
 */

/* Original file: SPRSCH.PAS */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/* Mixer spurious response search program */
/* Operation is self-explanatory */

#define FNSIZE 120

#ifndef true
# define true    1
# define false   0
#endif

#ifndef TRUE
# define TRUE    1
# define FALSE   0
#endif

typedef struct spurindex
{
  short m, n;
}
spurindex;


static short spurnum, maxspurnum, m, n, mmax, nmax;
static double frfmax, frfmin, fifmax, fifmin, flomax, flomin, fifmaxm, fifminm, smin, smax, slope, x;
static spurindex mnpair[100];
static spurindex pair;
static double frftop[100], frfbot[100];
static char filename[81];
static FILE *outfile;
static char response;
static char outfile_NAME[FNSIZE];


static unsigned char In_interval(double low, double x, double high)
{
  if (x >= low && x <= high)
    return true;
  else
    return false;
}


int main(int argc, char *argv[])
{
  char *TEMP;

  outfile = NULL;
  printf("                                    SPUR SEARCH\n\n");
  printf("                                     Ver.  I.1\n");
  printf("                                      3/13/85\n\n\n");
  printf("Enter maximum RF harmonic (M): ");
  scanf("%hi", &mmax);
  printf("Enter maximum LO harmonic (N): ");
  scanf("%hi", &nmax);
  printf("Enter lowest and highest RF frequency: ");
  scanf("%lf%lf", &frfmin, &frfmax);
  if (frfmax < frfmin) {
    x = frfmin;
    frfmin = frfmax;
    frfmax = x;
  }
  printf("Enter lowest and highest IF frequency: ");
  scanf("%lf %lf", &fifmin, &fifmax);
  if (fifmax < fifmin) {
    x = fifmin;
    fifmin = fifmax;
    fifmax = x;
  }
  printf("Enter lowest and highest LO frequency: ");
  scanf("%lf %lf", &flomin, &flomax);
  if (flomax < flomin) {
    x = flomin;
    flomin = flomax;
    flomax = x;
  }
  /* Begin the search */
  spurnum = 0;
  maxspurnum = 0;
  /* negative m */
  for (m = -mmax; m <= -1; m++) {
    fifmaxm = fifmax / m;
    fifminm = fifmin / m;
    smax = (frfmax - fifmaxm) / flomin;
    smin = (frfmin - fifminm) / flomax;
    for (n = -nmax; n <= nmax; n++) {
      slope = -((double) n / m);
      if (In_interval(smin, slope, smax)) {
	spurnum++;
	pair.n = n;
	pair.m = m;
	mnpair[spurnum - 1] = pair;
      }
    }
  }				/* m loop */
  /* positive m */
  for (m = 1; m <= mmax; m++) {
    fifmaxm = fifmax / m;
    fifminm = fifmin / m;
    if (fifmaxm > frfmin)
      smin = (frfmin - fifmaxm) / flomin;
    else
      smin = (frfmin - fifmaxm) / flomax;
    if (fifminm > frfmax)
      smax = (frfmax - fifminm) / flomax;
    else
      smax = (frfmax - fifminm) / flomin;
    for (n = -nmax; n <= nmax; n++) {
      slope = -((double) n / m);
      if (In_interval(smin, slope, smax)) {
	spurnum++;
	pair.n = n;
	pair.m = m;
	mnpair[spurnum - 1] = pair;
      }
    }
  }				/* m loop */
  /* find RF frequencies for the spurs */
  /* clear screen and write a message if no spurs were found */
  /* Clear_screen(); */
  maxspurnum = spurnum;
  if (maxspurnum == 0) {
    printf("\nNo spurious responses found for this set of frequencies\n\n");
    goto _L999;
  }
  /* if spurs were found, calculate frequencies and list them */
  //gotoxy(1, 1);
  printf("Spurious response indices and RF frequency ranges: \n\n");
  printf("RF range: %8.3f  to %8.3f\n", frfmin, frfmax);
  printf("IF range: %8.3f  to %8.3f\n", fifmin, fifmax);
  printf("LO range: %8.3f  to %8.3f\n", flomin, flomax);
  printf("Max RF harmonic (M): %2d    Max LO harmonic (N): %2d\n\n", mmax, nmax);
  printf("  M    N        f1           f2\n");
  printf("---------------------------------------\n");
  for (spurnum = 1; spurnum <= maxspurnum; spurnum++) {
    pair = mnpair[spurnum - 1];
    m = pair.m;
    n = pair.n;
    slope = -((double) n / m);
    fifmaxm = fifmax / m;
    fifminm = fifmin / m;
    if (m > 0 && slope >= 0) {
      frftop[spurnum - 1] = fifmaxm + slope * flomax;
      frfbot[spurnum - 1] = fifminm + slope * flomin;
    } else if (m > 0 && slope < 0) {
      frftop[spurnum - 1] = fifmaxm + slope * flomin;
      frfbot[spurnum - 1] = fifminm + slope * flomax;
    } else {
      frftop[spurnum - 1] = fifminm + slope * flomax;
      frfbot[spurnum - 1] = fifmaxm + slope * flomin;
    }
    printf("%3d%5d%12.3f%13.3f\n", m, n, frfbot[spurnum - 1], frftop[spurnum - 1]);
  }
  printf("\nWant to write results to a file? y/n: ");
  /* write results to a file if desired */
  /* read previous newline before reading response */
  scanf("\n%c", &response);
  if (response == 'y' || response == 'Y') {
    printf("\nEnter file name : ");
    /* read previous newline */
    getchar();
    /* read file name */
    fgets(filename, 81, stdin);
    TEMP = strchr(filename, '\n');
    if (TEMP != NULL)
      *TEMP = 0;
    if (strstr(filename, ".txt") == NULL && strstr(filename, ".TXT") == NULL) {
      strcat(filename, ".txt");
      /* possible string truncation in assignment */
    }
    strcpy(outfile_NAME, filename);
    if (outfile != NULL)
      outfile = freopen(outfile_NAME, "w", outfile);
    else
      outfile = fopen(outfile_NAME, "w");
    if (outfile == NULL) {
      fprintf(stderr, "error: %s: file not found\n", outfile_NAME);
      exit(-1);
    }
    fprintf(outfile, "RF range: %8.3f  to %8.3f\n", frfmin, frfmax);
    fprintf(outfile, "IF range: %8.3f  to %8.3f\n", fifmin, fifmax);
    fprintf(outfile, "LO range: %8.3f  to %8.3f\n", flomin, flomax);
    fprintf(outfile, "Max RF harmonic (M): %2d    Max LO harmonic (N): %2d\n\n", mmax, nmax);
    fprintf(outfile, "  M    N        f1           f2\n");
    fprintf(outfile, "--------------------------------------\n");
    for (spurnum = 1; spurnum <= maxspurnum; spurnum++) {
      pair = mnpair[spurnum - 1];
      m = pair.m;
      n = pair.n;
      fprintf(outfile, "%3d%5d%12.3f%13.3f\n", m, n, frfbot[spurnum - 1], frftop[spurnum - 1]);
    }
    if (outfile != NULL)
      fclose(outfile);
    outfile = NULL;
    printf("File %s written\n", filename);
  }
_L999:
  if (outfile != NULL)
    fclose(outfile);
  exit(0);
}
